using System;
using System.IO.Ports;

using Microsoft.SPOT;
using GHIElectronics.NETMF.Hardware;
using GHIElectronics.NETMF.Hardware.LowLevel;

namespace GHIElectronics.NETMF.FEZ
{
    public static partial class FEZ_Components
    {
        public class Fez_Xbee
        {
            static void RemapCOM4to_TXAn2_RXAn3(SerialPort ser)
            {
                // call this function **after** you open COM4 port
                if (ser.PortName != "COM4" || ser.IsOpen == false)
                    throw new Exception("Only use COM4 and make sure it is open");
                // remap COM4 RX (in) pin from P4.29/DIO17 to P0.26 (that is An3)
                // remap COM4 TX (out) pin from P4.28/DIO13 to P0.25 (that is An2)
                Register PINSEL9 = new Register(0xE002C024);
                PINSEL9.Write(0);// COM4 is now disconnected from P4.28 and P4.29
                Register PINSEL1 = new Register(0xE002C004);
                PINSEL1.SetBits(0xf << 18);// COM4 is now connected to An3 and An4
            }

            static SerialPort xbeePort = null;

            public static SerialPort GetPort(bool usingRobot)
            {
                if (xbeePort == null)
                {
                    if (usingRobot)
                    {
                        xbeePort = new SerialPort("COM4", 19200, Parity.None, 8, StopBits.One);
                        xbeePort.Open();
                        RemapCOM4to_TXAn2_RXAn3(xbeePort);
                    }
                    else
                    {
                        xbeePort = new SerialPort("COM2", 19200, Parity.None, 8, StopBits.One);
                        xbeePort.Open();
                    }
                }
                return xbeePort;
            }
        }
    }
}
